#include <Arduino.h>

/* Provide serial I/O methods */
class modem
{
public:
    // Transmit byte
    void SendByte(uint8_t byte)
    {
        Serial.write(byte);
    }
    // Test whether byte is incoming or not
    bool CharAvail()
    {
        return Serial.available() > 0;
    }
    // Test whether serial line is connected or not
    bool Carrier()
    {
        return Serial;
    }
    // Receive a byte, or give up within a time limit
    int GetByte(int tenths = 100)
    {
        Serial.setTimeout(tenths * 100);

        char byte = '?';
        int n = Serial.readBytes(&byte, 1);

        return (uint8_t) byte;
    }
};

/* Provide the ROM access methods */
class ROMsender: public modem
{
    /* Define the pins that the PROM chip is connected to: */
    enum { AddrLoBegin = 36, AddrHiBegin = 46, DataBegin = 26,
           CE = 2, VPP = 3 };

    // Poke the ROM address into the PROM chip's pins
    void SendAddress(uint32_t addr)
    {
        for(int a=0; a<8; ++a)
            { digitalWrite(AddrLoBegin + a, addr&1); addr>>=1; }
        for(int a=0; a<8; ++a)
            { digitalWrite(AddrHiBegin + a, addr&1); addr>>=1; }
        digitalWrite(CE,  addr&1); addr>>=1;
        digitalWrite(VPP, addr&1); addr>>=1;
    }
    // Read the ROM byte from the PROM chip's pins
    uint8_t ReadData()
    {
        uint8_t result = 0;
        for(int a=8; --a>=0; )
        {
            result <<= 1;
            result |= digitalRead(DataBegin+a);
        }
        return result;
    }
public:
    ROMsender()
    {
        // Configure the data I/O ports for input
        for(int a=0; a<8; ++a) pinMode(DataBegin+a, INPUT);
        // And the address I/O ports for output
        for(int a=0; a<8; ++a) pinMode(AddrLoBegin+a, OUTPUT);
        for(int a=0; a<8; ++a) pinMode(AddrHiBegin+a, OUTPUT);
        pinMode(VPP, OUTPUT); digitalWrite(VPP, LOW);
        pinMode(CE, OUTPUT);  digitalWrite(CE, LOW);
    }

    // Read ROM data from given position
    uint8_t GetData(uint32_t position)
    {
        SendAddress( position );
        return ReadData();
    }
};

void setup()
{
    // Start a serial port. We transmit the ROM data through the serial port.
    Serial.begin(115200);

    // Wait for the serial port to connect.
    while(!Serial) { }
}

void loop()
{
    // Wait until someone connects with a terminal program and hits "Y"
    unsigned long counter=0, counter2=0;
    for(;;)
    {
        if(Serial.available())
        {
            char c = Serial.read();
            if(c == 'Y' || c == 'y') break;
            continue;
        }
        if(counter++ == 1000)
        {
            char Buf[128];
            sprintf(Buf,
                "Asking for the #%lu time, "
                "are you ready to receive ROM dump? (Y/N)",
                ++counter2);
            Serial.println(Buf);
            counter = 0;
        }
        delay(2);
    }

    ROMsender sender;
    /* Send a sample of the ROM as a hex-dump */
    for(unsigned a=0; a<512; a+=16)
    {
        char data[16], Buf[5 + 16*3 + 16 + 1]="";
        sprintf(Buf, "%04X ", a);
        for(unsigned b=0; b<16; ++b)
            data[b] = sender.GetData(a+b);
        for(unsigned b=0; b<16; ++b)
            sprintf(Buf+5+b*3, "%02X%c", data[b], ((b+1)%8)?' ':'|');
        for(unsigned b=0; b<16; ++b)
            Buf[53+b] = uint8_t(data[b]-32) > (126-32) ? '.' : data[b];
        Serial.println(Buf);
    }

    Serial.println("Done!");
}



